﻿/**
* Vein by Grant Skinner. August 8, 2005
* Visit www.gskinner.com/blog for documentation, updates and more free code.
*
* You may distribute this code freely, as long as this comment block remains intact.
*/

import VeinManager;

class Vein {
// public properties:
	public static var MAX_THICKNESS=5;
	public var ID:String;
	
// private properties:
	private var x:Number=0;
	private var y:Number=0;
	private var angle:Number=0;
	private var thickness:Number=MAX_THICKNESS;
	private var originalThickness:Number;
	private var canvas:MovieClip;
	private var manager:VeinManager;
	private var count:Number=0;
	
// initialization:
	public function Vein(p_manager:VeinManager,p_canvas:MovieClip,p_thickness:Number,p_x:Number,p_y:Number,p_angle:Number) {
		// set up the initial properties:
		manager = p_manager;
		canvas = p_canvas;
		thickness = originalThickness = p_thickness;
		x = p_x;
		y = p_y;
		angle = p_angle;
	}
	
// public methods:
	// called once per frame by VeinManager:
	public function tick():Void {
		// start drawing from current position:
		canvas.moveTo(x,y);
		
		// this inverse thickness value is used frequently, so we'll pre-calculate it:
		var t:Number = MAX_THICKNESS-thickness+1;
		
		// do 
		var d:Number = Math.sqrt(x*x+y*y);
		if (thickness == MAX_THICKNESS && d > random(700)) {
			// if this is an artery, and it's gotten too far away from the center, then we need to angle back:
			var a:Number = (Math.atan2(y,x)/Math.PI*180+180)%360;
			var ad:Number = (a+720)-(angle*180/Math.PI+720);
			
			// account for passing 0:
			if (ad > 180) { ad -= 360; }
			else if (ad < -180) { ad += 360; }
			
			angle += Math.min(10,Math.max(-10,ad/(random(30)+3)))/180*Math.PI;
		} else {
			// turn slightly a random amount (thinner veins turn more):
			angle += Math.PI/180*(random(t*16+10)-(t*8+5));
		}
		
		// choose a random length for the new segment, and calculate the new position:
		var length:Number = random(5)+t+4;
		x += length*Math.cos(angle);
		y += length*Math.sin(angle);
		
		// draw a line to the new position:
		canvas.lineStyle(thickness/1.5,0xFFFFFF-t*0x301000,50+thickness/MAX_THICKNESS*50);
		canvas.lineTo(x,y);
		
		// create children randomly, with a lower thickness, and the same initial x,y,angle:
		if (thickness > 1 && random(((thickness==MAX_THICKNESS) ? 7 : 2*thickness)) == 0) {
			manager.createVein(thickness-1-random(thickness-1),x,y,angle);
		}
		
		// randomly reduce the thickness (so that the vein tapers off)
		if (thickness < MAX_THICKNESS && count > random(14)*t) {
			count = 0;
			thickness--;
			// remove this vein if it's thickness is 0:
			if (thickness == 0) { manager.removeVein(this); }
		} else {
			count++;
		}
	}
}