﻿/**
* VeinManager by Grant Skinner. August 8, 2005
* Visit www.gskinner.com/blog for documentation, updates and more free code.
*
* You may distribute this code freely, as long as this comment block remains intact.
*/

import Vein;
import flash.display.BitmapData;
import flash.filters.BlurFilter;
import flash.filters.ColorMatrixFilter;
import flash.geom.Matrix;
import flash.geom.Rectangle;
import flash.geom.Point;

class VeinManager extends MovieClip {
// private properties:
	private static var WIDTH:Number=460;
	private static var HEIGHT:Number=460;
	
	private var veins:Object; // hashtable of active veins
	private var nextVeinID:Number=0;
	
	private var blurF:BlurFilter;
	private var colorF:ColorMatrixFilter;
	
	private var mat:Matrix;
	private var rect:Rectangle;
	private var pnt:Point;
	private var count:Number=0;
	
// ui elements:
	private var canvas:MovieClip;
	private var canvasBmp:BitmapData;
	
// initialization:
	function VeinManager() {
		// set up all the stage elements:
		createEmptyMovieClip("canvas",0);
		canvas.blendMode = "add";
		canvas.createEmptyMovieClip("image",0);
		canvas.createEmptyMovieClip("veins",2);
		
		// center the canvas:
		canvas._x = WIDTH/2;
		canvas._y = HEIGHT/2;
		// counter-offset the bitmap holder:
		canvas.image._x = -canvas._x;
		canvas.image._y = -canvas._y;
		
		// set up the bitmap:
		canvasBmp = new BitmapData(WIDTH,HEIGHT,true,0);
		canvas.image.attachBitmap(canvasBmp,1);
		
		// set up the filters:
		colorF = new ColorMatrixFilter([
			0.97,0,0,0,2,
			0,0.99,0,0,3,
			0,0,0.98,0,4,
			0,0,0,1,-0.7
		]);
		blurF = new BlurFilter(2,2,1);
		
		// set up the matrix that will be used to draw the veins onto the bitmap:
		// it requires the same offset as canvas, so we can just use its matrix:
		mat = canvas.transform.matrix;
		
		// set up the rect for filters, describes the boundaries of the bitmap:
		rect = new Rectangle(0,0,WIDTH,HEIGHT);
		// set up the origin point for filters:
		pnt = new Point(0,0);
		
		// set up the veins hashtable:
		veins = {};
		// create the initial arteries, starting from the center, with random angles:
		createVein(Vein.MAX_THICKNESS,0,0,random(6));
		createVein(Vein.MAX_THICKNESS,0,0,random(6));
		createVein(Vein.MAX_THICKNESS,0,0,random(6));
		
		// call tick once per frame:
		onEnterFrame = tick;
	}
	
// public methods:
	// called by veins when they are ready to be removed:
	public function removeVein(p_vein:Vein):Void {
		// simply remove the reference to the vein from the hashtable:
		delete(veins[p_vein.ID]);
	}
	
	// creates new veins:
	public function createVein(p_thickness:Number,p_x:Number,p_y:Number,p_angle:Number):Void {
		var v:Vein = new Vein(this,canvas.veins,p_thickness,p_x,p_y,p_angle);
		v.ID = "v"+(++nextVeinID);
		// add the new vein to the hashtable:
		veins[v.ID] = v;
	}
	
// private methods:
	// called once each frame:
	public function tick():Void {
		// this is just to work around some issues in the display framework:
		var p:MovieClip = this;
		while (p._parent != undefined) {
			if (!p._visible||p._visible==undefined) { delete(onEnterFrame); return; }
			p = p._parent;
		}
		
		// tick each of the veins:
		for (var n:String in veins) {
			veins[n].tick();
		}
		
		// blit the vectors to the bitmap:
		canvasBmp.draw(canvas.veins,mat);
		// clear the vectors:
		canvas.veins.clear();
		
		// execute every 5 frames:
		if (++count%5 == 0) {
			// apply the filters:
			canvasBmp.applyFilter(canvasBmp,rect,pnt,colorF);
			canvasBmp.applyFilter(canvasBmp,rect,pnt,blurF);
		}
	}
	
	
	
}